// SPDX-License-Identifier: BSD-3-Clause
/* Copyright(c) 2014-2024 Broadcom
 * All rights reserved.
 */

#include "ulp_template_db_enum.h"
#include "ulp_template_db_field.h"
#include "ulp_template_struct.h"
#include "ulp_template_db_tbl.h"

/* Define the template match patterns */
/*
 * List of protocol matches
 */
struct bnxt_ulp_class_match_info ulp_class_match_list[] = {
	[1] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB800000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		},
	},
	[2] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA00000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[3] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA00000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[4] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA40000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[5] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA78000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		},
	},
	[6] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA36000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		},
	},
	[7] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4F000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[8] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46C00000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[9] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4F000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[10] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46C00000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[11] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA49E00000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 11,
		[58] = 12,
		[60] = 13,
		[62] = 14,
		[64] = 15,
		[66] = 16,
		[68] = 17,
		[70] = 18,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[12] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA48D80000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 11,
		[38] = 12,
		[40] = 13,
		[42] = 14,
		[44] = 15,
		[46] = 16,
		[48] = 17,
		[50] = 18,
		[52] = 19,
		[54] = 20,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[13] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBE20000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 5,
		[90] = 6,
		[92] = 7,
		[94] = 8,
		[96] = 9,
		[98] = 10,
		[100] = 11,
		[102] = 12,
		[104] = 13,
		},
	},
	[14] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBE00000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 5,
		[108] = 6,
		[110] = 7,
		[112] = 8,
		},
	},
	[15] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC4000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 8,
		[90] = 9,
		[92] = 10,
		[94] = 11,
		[96] = 12,
		[98] = 13,
		[100] = 14,
		[102] = 15,
		[104] = 16,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[16] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC0000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 8,
		[108] = 9,
		[110] = 10,
		[112] = 11,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[17] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC4000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 8,
		[90] = 9,
		[92] = 10,
		[94] = 11,
		[96] = 12,
		[98] = 13,
		[100] = 14,
		[102] = 15,
		[104] = 16,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[18] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC0000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 8,
		[108] = 9,
		[110] = 10,
		[112] = 11,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[19] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA58800000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 11,
		[90] = 12,
		[92] = 13,
		[94] = 14,
		[96] = 15,
		[98] = 16,
		[100] = 17,
		[102] = 18,
		[104] = 19,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[20] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA58000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 11,
		[108] = 12,
		[110] = 13,
		[112] = 14,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[21] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA7E200000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[88] = 13,
		[90] = 14,
		[92] = 15,
		[94] = 16,
		[96] = 17,
		[98] = 18,
		[100] = 19,
		[102] = 20,
		[104] = 21,
		},
	},
	[22] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA37880000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[88] = 15,
		[90] = 16,
		[92] = 17,
		[94] = 18,
		[96] = 19,
		[98] = 20,
		[100] = 21,
		[102] = 22,
		[104] = 23,
		},
	},
	[23] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA7E000000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		},
	},
	[24] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA37800000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		},
	},
	[25] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC40000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[88] = 16,
		[90] = 17,
		[92] = 18,
		[94] = 19,
		[96] = 20,
		[98] = 21,
		[100] = 22,
		[102] = 23,
		[104] = 24,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[26] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F10000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[88] = 18,
		[90] = 19,
		[92] = 20,
		[94] = 21,
		[96] = 22,
		[98] = 23,
		[100] = 24,
		[102] = 25,
		[104] = 26,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[27] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC00000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[106] = 16,
		[108] = 17,
		[110] = 18,
		[112] = 19,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[28] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F00000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[106] = 18,
		[108] = 19,
		[110] = 20,
		[112] = 21,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[29] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC40000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[88] = 16,
		[90] = 17,
		[92] = 18,
		[94] = 19,
		[96] = 20,
		[98] = 21,
		[100] = 22,
		[102] = 23,
		[104] = 24,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[30] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F10000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[88] = 18,
		[90] = 19,
		[92] = 20,
		[94] = 21,
		[96] = 22,
		[98] = 23,
		[100] = 24,
		[102] = 25,
		[104] = 26,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[31] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC00000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[106] = 16,
		[108] = 17,
		[110] = 18,
		[112] = 19,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[32] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F00000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[106] = 18,
		[108] = 19,
		[110] = 20,
		[112] = 21,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[33] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA49F88000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 11,
		[58] = 12,
		[60] = 13,
		[62] = 14,
		[64] = 15,
		[66] = 16,
		[68] = 17,
		[70] = 18,
		[88] = 19,
		[90] = 20,
		[92] = 21,
		[94] = 22,
		[96] = 23,
		[98] = 24,
		[100] = 25,
		[102] = 26,
		[104] = 27,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[34] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA48DE2000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 11,
		[38] = 12,
		[40] = 13,
		[42] = 14,
		[44] = 15,
		[46] = 16,
		[48] = 17,
		[50] = 18,
		[52] = 19,
		[54] = 20,
		[88] = 21,
		[90] = 22,
		[92] = 23,
		[94] = 24,
		[96] = 25,
		[98] = 26,
		[100] = 27,
		[102] = 28,
		[104] = 29,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[35] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA49F80000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 11,
		[58] = 12,
		[60] = 13,
		[62] = 14,
		[64] = 15,
		[66] = 16,
		[68] = 17,
		[70] = 18,
		[106] = 19,
		[108] = 20,
		[110] = 21,
		[112] = 22,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[36] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA48DE0000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 11,
		[38] = 12,
		[40] = 13,
		[42] = 14,
		[44] = 15,
		[46] = 16,
		[48] = 17,
		[50] = 18,
		[52] = 19,
		[54] = 20,
		[106] = 21,
		[108] = 22,
		[110] = 23,
		[112] = 24,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[37] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A170000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[38] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685C000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[39] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A170B00000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[57] = 24,
		[58] = 6,
		[59] = 25,
		[60] = 7,
		[61] = 26,
		[62] = 8,
		[63] = 27,
		[64] = 9,
		[65] = 28,
		[66] = 10,
		[67] = 29,
		[68] = 11,
		[69] = 30,
		[70] = 12,
		[71] = 31,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[40] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685C2C0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[57] = 26,
		[59] = 27,
		[61] = 28,
		[63] = 29,
		[65] = 30,
		[67] = 31,
		[69] = 32,
		[71] = 33,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[41] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A1702C0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[37] = 24,
		[39] = 25,
		[41] = 26,
		[43] = 27,
		[45] = 28,
		[47] = 29,
		[49] = 30,
		[51] = 31,
		[53] = 32,
		[55] = 33,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[42] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685C0B0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[37] = 26,
		[38] = 6,
		[39] = 27,
		[40] = 7,
		[41] = 28,
		[42] = 8,
		[43] = 29,
		[44] = 9,
		[45] = 30,
		[46] = 10,
		[47] = 31,
		[48] = 11,
		[49] = 32,
		[50] = 12,
		[51] = 33,
		[52] = 13,
		[53] = 34,
		[54] = 14,
		[55] = 35,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[43] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A17C000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[89] = 24,
		[91] = 25,
		[93] = 26,
		[95] = 27,
		[97] = 28,
		[99] = 29,
		[101] = 30,
		[103] = 31,
		[105] = 32,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[44] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685F000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[89] = 26,
		[91] = 27,
		[93] = 28,
		[95] = 29,
		[97] = 30,
		[99] = 31,
		[101] = 32,
		[103] = 33,
		[105] = 34,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[45] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A17C000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[107] = 24,
		[108] = 14,
		[109] = 25,
		[110] = 15,
		[111] = 26,
		[112] = 16,
		[113] = 27,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[46] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685F000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[107] = 26,
		[108] = 16,
		[109] = 27,
		[110] = 17,
		[111] = 28,
		[112] = 18,
		[113] = 29,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[47] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A170BC0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[57] = 24,
		[58] = 6,
		[59] = 25,
		[60] = 7,
		[61] = 26,
		[62] = 8,
		[63] = 27,
		[64] = 9,
		[65] = 28,
		[66] = 10,
		[67] = 29,
		[68] = 11,
		[69] = 30,
		[70] = 12,
		[71] = 31,
		[89] = 32,
		[91] = 33,
		[93] = 34,
		[95] = 35,
		[97] = 36,
		[99] = 37,
		[101] = 38,
		[103] = 39,
		[105] = 40,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[48] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685C2F0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[57] = 26,
		[59] = 27,
		[61] = 28,
		[63] = 29,
		[65] = 30,
		[67] = 31,
		[69] = 32,
		[71] = 33,
		[89] = 34,
		[91] = 35,
		[93] = 36,
		[95] = 37,
		[97] = 38,
		[99] = 39,
		[101] = 40,
		[103] = 41,
		[105] = 42,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[49] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A1702F0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[37] = 24,
		[39] = 25,
		[41] = 26,
		[43] = 27,
		[45] = 28,
		[47] = 29,
		[49] = 30,
		[51] = 31,
		[53] = 32,
		[55] = 33,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[89] = 34,
		[91] = 35,
		[93] = 36,
		[95] = 37,
		[97] = 38,
		[99] = 39,
		[101] = 40,
		[103] = 41,
		[105] = 42,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[50] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685C0BC000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[37] = 26,
		[38] = 6,
		[39] = 27,
		[40] = 7,
		[41] = 28,
		[42] = 8,
		[43] = 29,
		[44] = 9,
		[45] = 30,
		[46] = 10,
		[47] = 31,
		[48] = 11,
		[49] = 32,
		[50] = 12,
		[51] = 33,
		[52] = 13,
		[53] = 34,
		[54] = 14,
		[55] = 35,
		[89] = 36,
		[91] = 37,
		[93] = 38,
		[95] = 39,
		[97] = 40,
		[99] = 41,
		[101] = 42,
		[103] = 43,
		[105] = 44,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[51] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A170BC0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[57] = 24,
		[58] = 6,
		[59] = 25,
		[60] = 7,
		[61] = 26,
		[62] = 8,
		[63] = 27,
		[64] = 9,
		[65] = 28,
		[66] = 10,
		[67] = 29,
		[68] = 11,
		[69] = 30,
		[70] = 12,
		[71] = 31,
		[106] = 13,
		[107] = 32,
		[108] = 14,
		[109] = 33,
		[110] = 15,
		[111] = 34,
		[112] = 16,
		[113] = 35,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[52] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685C2F0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[57] = 26,
		[59] = 27,
		[61] = 28,
		[63] = 29,
		[65] = 30,
		[67] = 31,
		[69] = 32,
		[71] = 33,
		[106] = 15,
		[107] = 34,
		[108] = 16,
		[109] = 35,
		[110] = 17,
		[111] = 36,
		[112] = 18,
		[113] = 37,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[53] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB01A1702F0000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[37] = 24,
		[39] = 25,
		[41] = 26,
		[43] = 27,
		[45] = 28,
		[47] = 29,
		[49] = 30,
		[51] = 31,
		[53] = 32,
		[55] = 33,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[107] = 34,
		[108] = 14,
		[109] = 35,
		[110] = 15,
		[111] = 36,
		[112] = 16,
		[113] = 37,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[54] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00685C0BC000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[37] = 26,
		[38] = 6,
		[39] = 27,
		[40] = 7,
		[41] = 28,
		[42] = 8,
		[43] = 29,
		[44] = 9,
		[45] = 30,
		[46] = 10,
		[47] = 31,
		[48] = 11,
		[49] = 32,
		[50] = 12,
		[51] = 33,
		[52] = 13,
		[53] = 34,
		[54] = 14,
		[55] = 35,
		[106] = 15,
		[107] = 36,
		[108] = 16,
		[109] = 37,
		[110] = 17,
		[111] = 38,
		[112] = 18,
		[113] = 39,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[55] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F1 |
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x200A000000000000,
	.field_opt_bitmap = 0x9000000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 2,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[56] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F1 |
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x2002800000000000,
	.field_opt_bitmap = 0x9000000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 2,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[57] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0xC0800000000000,
	.field_opt_bitmap = 0x8010301800000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 18,
		[13] = 19,
		[15] = 20,
		[56] = 2,
		[57] = 21,
		[58] = 3,
		[59] = 22,
		[60] = 4,
		[61] = 23,
		[62] = 5,
		[63] = 24,
		[64] = 6,
		[65] = 25,
		[66] = 7,
		[67] = 26,
		[68] = 8,
		[69] = 27,
		[70] = 9,
		[71] = 28,
		[106] = 10,
		[108] = 11,
		[110] = 12,
		[112] = 13,
		[126] = 14,
		[127] = 15,
		[128] = 16,
		[129] = 17,
		},
	},
	[58] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x30200000000000,
	.field_opt_bitmap = 0x80040C0600000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 20,
		[13] = 21,
		[15] = 22,
		[36] = 2,
		[38] = 3,
		[40] = 4,
		[42] = 5,
		[44] = 6,
		[46] = 7,
		[48] = 8,
		[50] = 9,
		[52] = 10,
		[54] = 11,
		[57] = 23,
		[59] = 24,
		[61] = 25,
		[63] = 26,
		[65] = 27,
		[67] = 28,
		[69] = 29,
		[71] = 30,
		[106] = 12,
		[108] = 13,
		[110] = 14,
		[112] = 15,
		[126] = 16,
		[127] = 17,
		[128] = 18,
		[129] = 19,
		},
	},
	[59] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0xC0800000000000,
	.field_opt_bitmap = 0x8010300600000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 18,
		[13] = 19,
		[15] = 20,
		[37] = 21,
		[39] = 22,
		[41] = 23,
		[43] = 24,
		[45] = 25,
		[47] = 26,
		[49] = 27,
		[51] = 28,
		[53] = 29,
		[55] = 30,
		[56] = 2,
		[58] = 3,
		[60] = 4,
		[62] = 5,
		[64] = 6,
		[66] = 7,
		[68] = 8,
		[70] = 9,
		[106] = 10,
		[108] = 11,
		[110] = 12,
		[112] = 13,
		[126] = 14,
		[127] = 15,
		[128] = 16,
		[129] = 17,
		},
	},
	[60] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x30200000000000,
	.field_opt_bitmap = 0x80040C0180000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 20,
		[13] = 21,
		[15] = 22,
		[36] = 2,
		[37] = 23,
		[38] = 3,
		[39] = 24,
		[40] = 4,
		[41] = 25,
		[42] = 5,
		[43] = 26,
		[44] = 6,
		[45] = 27,
		[46] = 7,
		[47] = 28,
		[48] = 8,
		[49] = 29,
		[50] = 9,
		[51] = 30,
		[52] = 10,
		[53] = 31,
		[54] = 11,
		[55] = 32,
		[106] = 12,
		[108] = 13,
		[110] = 14,
		[112] = 15,
		[126] = 16,
		[127] = 17,
		[128] = 18,
		[129] = 19,
		},
	},
	[61] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0xC0800000000000,
	.field_opt_bitmap = 0x8010301E00000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 18,
		[13] = 19,
		[15] = 20,
		[56] = 2,
		[57] = 21,
		[58] = 3,
		[59] = 22,
		[60] = 4,
		[61] = 23,
		[62] = 5,
		[63] = 24,
		[64] = 6,
		[65] = 25,
		[66] = 7,
		[67] = 26,
		[68] = 8,
		[69] = 27,
		[70] = 9,
		[71] = 28,
		[89] = 29,
		[91] = 30,
		[93] = 31,
		[95] = 32,
		[97] = 33,
		[99] = 34,
		[101] = 35,
		[103] = 36,
		[105] = 37,
		[106] = 10,
		[108] = 11,
		[110] = 12,
		[112] = 13,
		[126] = 14,
		[127] = 15,
		[128] = 16,
		[129] = 17,
		},
	},
	[62] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x30200000000000,
	.field_opt_bitmap = 0x80040C0780000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 20,
		[13] = 21,
		[15] = 22,
		[36] = 2,
		[38] = 3,
		[40] = 4,
		[42] = 5,
		[44] = 6,
		[46] = 7,
		[48] = 8,
		[50] = 9,
		[52] = 10,
		[54] = 11,
		[57] = 23,
		[59] = 24,
		[61] = 25,
		[63] = 26,
		[65] = 27,
		[67] = 28,
		[69] = 29,
		[71] = 30,
		[89] = 31,
		[91] = 32,
		[93] = 33,
		[95] = 34,
		[97] = 35,
		[99] = 36,
		[101] = 37,
		[103] = 38,
		[105] = 39,
		[106] = 12,
		[108] = 13,
		[110] = 14,
		[112] = 15,
		[126] = 16,
		[127] = 17,
		[128] = 18,
		[129] = 19,
		},
	},
	[63] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0xC0800000000000,
	.field_opt_bitmap = 0x8010300780000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 18,
		[13] = 19,
		[15] = 20,
		[37] = 21,
		[39] = 22,
		[41] = 23,
		[43] = 24,
		[45] = 25,
		[47] = 26,
		[49] = 27,
		[51] = 28,
		[53] = 29,
		[55] = 30,
		[56] = 2,
		[58] = 3,
		[60] = 4,
		[62] = 5,
		[64] = 6,
		[66] = 7,
		[68] = 8,
		[70] = 9,
		[89] = 31,
		[91] = 32,
		[93] = 33,
		[95] = 34,
		[97] = 35,
		[99] = 36,
		[101] = 37,
		[103] = 38,
		[105] = 39,
		[106] = 10,
		[108] = 11,
		[110] = 12,
		[112] = 13,
		[126] = 14,
		[127] = 15,
		[128] = 16,
		[129] = 17,
		},
	},
	[64] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x30200000000000,
	.field_opt_bitmap = 0x80040C01E0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 20,
		[13] = 21,
		[15] = 22,
		[36] = 2,
		[37] = 23,
		[38] = 3,
		[39] = 24,
		[40] = 4,
		[41] = 25,
		[42] = 5,
		[43] = 26,
		[44] = 6,
		[45] = 27,
		[46] = 7,
		[47] = 28,
		[48] = 8,
		[49] = 29,
		[50] = 9,
		[51] = 30,
		[52] = 10,
		[53] = 31,
		[54] = 11,
		[55] = 32,
		[89] = 33,
		[91] = 34,
		[93] = 35,
		[95] = 36,
		[97] = 37,
		[99] = 38,
		[101] = 39,
		[103] = 40,
		[105] = 41,
		[106] = 12,
		[108] = 13,
		[110] = 14,
		[112] = 15,
		[126] = 16,
		[127] = 17,
		[128] = 18,
		[129] = 19,
		},
	},
	[65] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0xC0800000000000,
	.field_opt_bitmap = 0x8010301E00000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 18,
		[13] = 19,
		[15] = 20,
		[56] = 2,
		[57] = 21,
		[58] = 3,
		[59] = 22,
		[60] = 4,
		[61] = 23,
		[62] = 5,
		[63] = 24,
		[64] = 6,
		[65] = 25,
		[66] = 7,
		[67] = 26,
		[68] = 8,
		[69] = 27,
		[70] = 9,
		[71] = 28,
		[106] = 10,
		[107] = 29,
		[108] = 11,
		[109] = 30,
		[110] = 12,
		[111] = 31,
		[112] = 13,
		[113] = 32,
		[126] = 14,
		[127] = 15,
		[128] = 16,
		[129] = 17,
		},
	},
	[66] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x30200000000000,
	.field_opt_bitmap = 0x80040C0780000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 20,
		[13] = 21,
		[15] = 22,
		[36] = 2,
		[38] = 3,
		[40] = 4,
		[42] = 5,
		[44] = 6,
		[46] = 7,
		[48] = 8,
		[50] = 9,
		[52] = 10,
		[54] = 11,
		[57] = 23,
		[59] = 24,
		[61] = 25,
		[63] = 26,
		[65] = 27,
		[67] = 28,
		[69] = 29,
		[71] = 30,
		[106] = 12,
		[107] = 31,
		[108] = 13,
		[109] = 32,
		[110] = 14,
		[111] = 33,
		[112] = 15,
		[113] = 34,
		[126] = 16,
		[127] = 17,
		[128] = 18,
		[129] = 19,
		},
	},
	[67] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0xC0800000000000,
	.field_opt_bitmap = 0x8010300780000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 18,
		[13] = 19,
		[15] = 20,
		[37] = 21,
		[39] = 22,
		[41] = 23,
		[43] = 24,
		[45] = 25,
		[47] = 26,
		[49] = 27,
		[51] = 28,
		[53] = 29,
		[55] = 30,
		[56] = 2,
		[58] = 3,
		[60] = 4,
		[62] = 5,
		[64] = 6,
		[66] = 7,
		[68] = 8,
		[70] = 9,
		[106] = 10,
		[107] = 31,
		[108] = 11,
		[109] = 32,
		[110] = 12,
		[111] = 33,
		[112] = 13,
		[113] = 34,
		[126] = 14,
		[127] = 15,
		[128] = 16,
		[129] = 17,
		},
	},
	[68] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x30200000000000,
	.field_opt_bitmap = 0x80040C01E0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		[11] = 20,
		[13] = 21,
		[15] = 22,
		[36] = 2,
		[37] = 23,
		[38] = 3,
		[39] = 24,
		[40] = 4,
		[41] = 25,
		[42] = 5,
		[43] = 26,
		[44] = 6,
		[45] = 27,
		[46] = 7,
		[47] = 28,
		[48] = 8,
		[49] = 29,
		[50] = 9,
		[51] = 30,
		[52] = 10,
		[53] = 31,
		[54] = 11,
		[55] = 32,
		[106] = 12,
		[107] = 33,
		[108] = 13,
		[109] = 34,
		[110] = 14,
		[111] = 35,
		[112] = 15,
		[113] = 36,
		[126] = 16,
		[127] = 17,
		[128] = 18,
		[129] = 19,
		},
	},
	[69] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_ICMP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0xC0800000000000,
	.field_opt_bitmap = 0x8010300600000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 4,
	.field_list = {
		[1] = 1,
		[11] = 18,
		[13] = 19,
		[15] = 20,
		[27] = 31,
		[29] = 32,
		[31] = 33,
		[33] = 34,
		[35] = 35,
		[37] = 21,
		[39] = 22,
		[41] = 23,
		[43] = 24,
		[45] = 25,
		[47] = 26,
		[49] = 27,
		[51] = 28,
		[53] = 29,
		[55] = 30,
		[56] = 2,
		[58] = 3,
		[60] = 4,
		[62] = 5,
		[64] = 6,
		[66] = 7,
		[68] = 8,
		[70] = 9,
		[106] = 10,
		[108] = 11,
		[110] = 12,
		[112] = 13,
		[126] = 14,
		[127] = 15,
		[128] = 16,
		[129] = 17,
		},
	},
	[70] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_F2 |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_ICMP |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x30200000000000,
	.field_opt_bitmap = 0x80040C0180000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 4,
	.field_list = {
		[1] = 1,
		[11] = 20,
		[13] = 21,
		[15] = 22,
		[27] = 33,
		[29] = 34,
		[31] = 35,
		[33] = 36,
		[35] = 37,
		[36] = 2,
		[37] = 23,
		[38] = 3,
		[39] = 24,
		[40] = 4,
		[41] = 25,
		[42] = 5,
		[43] = 26,
		[44] = 6,
		[45] = 27,
		[46] = 7,
		[47] = 28,
		[48] = 8,
		[49] = 29,
		[50] = 9,
		[51] = 30,
		[52] = 10,
		[53] = 31,
		[54] = 11,
		[55] = 32,
		[106] = 12,
		[108] = 13,
		[110] = 14,
		[112] = 15,
		[126] = 16,
		[127] = 17,
		[128] = 18,
		[129] = 19,
		},
	},
	[71] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_GENEVE |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xA002800000000000,
	.field_exclude_bitmap = 0x2000000000000000,
	.class_tid = 1,
	.flow_pattern_id = 5,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		},
	},
	[72] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_SVIF |
		BNXT_ULP_FLOW_DIR_BITMASK_ING },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0x8000000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 1,
	.flow_pattern_id = 6,
	.field_list = {
		[1] = 1,
		},
	},
	[73] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB800000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		},
	},
	[74] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA00000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[75] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA00000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[76] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA40000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[77] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA78000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		},
	},
	[78] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA36000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		},
	},
	[79] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4F000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[80] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46C00000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[81] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4F000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[82] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46C00000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[83] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA49E00000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 11,
		[58] = 12,
		[60] = 13,
		[62] = 14,
		[64] = 15,
		[66] = 16,
		[68] = 17,
		[70] = 18,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[84] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA48D80000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 11,
		[38] = 12,
		[40] = 13,
		[42] = 14,
		[44] = 15,
		[46] = 16,
		[48] = 17,
		[50] = 18,
		[52] = 19,
		[54] = 20,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[85] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBE20000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 5,
		[90] = 6,
		[92] = 7,
		[94] = 8,
		[96] = 9,
		[98] = 10,
		[100] = 11,
		[102] = 12,
		[104] = 13,
		},
	},
	[86] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBE00000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 5,
		[108] = 6,
		[110] = 7,
		[112] = 8,
		},
	},
	[87] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC4000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 8,
		[90] = 9,
		[92] = 10,
		[94] = 11,
		[96] = 12,
		[98] = 13,
		[100] = 14,
		[102] = 15,
		[104] = 16,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[88] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC0000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 8,
		[108] = 9,
		[110] = 10,
		[112] = 11,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[89] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC4000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 8,
		[90] = 9,
		[92] = 10,
		[94] = 11,
		[96] = 12,
		[98] = 13,
		[100] = 14,
		[102] = 15,
		[104] = 16,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[90] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBAC0000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 8,
		[108] = 9,
		[110] = 10,
		[112] = 11,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[91] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA58800000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[88] = 11,
		[90] = 12,
		[92] = 13,
		[94] = 14,
		[96] = 15,
		[98] = 16,
		[100] = 17,
		[102] = 18,
		[104] = 19,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[92] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA58000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[106] = 11,
		[108] = 12,
		[110] = 13,
		[112] = 14,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[93] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA7E200000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[88] = 13,
		[90] = 14,
		[92] = 15,
		[94] = 16,
		[96] = 17,
		[98] = 18,
		[100] = 19,
		[102] = 20,
		[104] = 21,
		},
	},
	[94] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA37880000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[88] = 15,
		[90] = 16,
		[92] = 17,
		[94] = 18,
		[96] = 19,
		[98] = 20,
		[100] = 21,
		[102] = 22,
		[104] = 23,
		},
	},
	[95] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA7E000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		},
	},
	[96] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA37800000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		},
	},
	[97] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC40000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[88] = 16,
		[90] = 17,
		[92] = 18,
		[94] = 19,
		[96] = 20,
		[98] = 21,
		[100] = 22,
		[102] = 23,
		[104] = 24,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[98] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F10000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[88] = 18,
		[90] = 19,
		[92] = 20,
		[94] = 21,
		[96] = 22,
		[98] = 23,
		[100] = 24,
		[102] = 25,
		[104] = 26,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[99] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC00000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[106] = 16,
		[108] = 17,
		[110] = 18,
		[112] = 19,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[100] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F00000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[106] = 18,
		[108] = 19,
		[110] = 20,
		[112] = 21,
		[114] = 5,
		[118] = 6,
		[122] = 7,
		},
	},
	[101] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC40000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[88] = 16,
		[90] = 17,
		[92] = 18,
		[94] = 19,
		[96] = 20,
		[98] = 21,
		[100] = 22,
		[102] = 23,
		[104] = 24,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[102] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F10000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[88] = 18,
		[90] = 19,
		[92] = 20,
		[94] = 21,
		[96] = 22,
		[98] = 23,
		[100] = 24,
		[102] = 25,
		[104] = 26,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[103] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA4FC00000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 8,
		[58] = 9,
		[60] = 10,
		[62] = 11,
		[64] = 12,
		[66] = 13,
		[68] = 14,
		[70] = 15,
		[106] = 16,
		[108] = 17,
		[110] = 18,
		[112] = 19,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[104] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA46F00000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 8,
		[38] = 9,
		[40] = 10,
		[42] = 11,
		[44] = 12,
		[46] = 13,
		[48] = 14,
		[50] = 15,
		[52] = 16,
		[54] = 17,
		[106] = 18,
		[108] = 19,
		[110] = 20,
		[112] = 21,
		[115] = 5,
		[119] = 6,
		[123] = 7,
		},
	},
	[105] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA49F88000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 11,
		[58] = 12,
		[60] = 13,
		[62] = 14,
		[64] = 15,
		[66] = 16,
		[68] = 17,
		[70] = 18,
		[88] = 19,
		[90] = 20,
		[92] = 21,
		[94] = 22,
		[96] = 23,
		[98] = 24,
		[100] = 25,
		[102] = 26,
		[104] = 27,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[106] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA48DE2000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 11,
		[38] = 12,
		[40] = 13,
		[42] = 14,
		[44] = 15,
		[46] = 16,
		[48] = 17,
		[50] = 18,
		[52] = 19,
		[54] = 20,
		[88] = 21,
		[90] = 22,
		[92] = 23,
		[94] = 24,
		[96] = 25,
		[98] = 26,
		[100] = 27,
		[102] = 28,
		[104] = 29,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[107] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA49F80000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[56] = 11,
		[58] = 12,
		[60] = 13,
		[62] = 14,
		[64] = 15,
		[66] = 16,
		[68] = 17,
		[70] = 18,
		[106] = 19,
		[108] = 20,
		[110] = 21,
		[112] = 22,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[108] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_OO_VLAN |
		BNXT_ULP_HDR_BIT_OI_VLAN |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xBA48DE0000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 0,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 11,
		[38] = 12,
		[40] = 13,
		[42] = 14,
		[44] = 15,
		[46] = 16,
		[48] = 17,
		[50] = 18,
		[52] = 19,
		[54] = 20,
		[106] = 21,
		[108] = 22,
		[110] = 23,
		[112] = 24,
		[114] = 5,
		[115] = 8,
		[118] = 6,
		[119] = 9,
		[122] = 7,
		[123] = 10,
		},
	},
	[109] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A170000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[110] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285C000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[111] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A170B00000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[57] = 24,
		[58] = 6,
		[59] = 25,
		[60] = 7,
		[61] = 26,
		[62] = 8,
		[63] = 27,
		[64] = 9,
		[65] = 28,
		[66] = 10,
		[67] = 29,
		[68] = 11,
		[69] = 30,
		[70] = 12,
		[71] = 31,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[112] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285C2C0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[57] = 26,
		[59] = 27,
		[61] = 28,
		[63] = 29,
		[65] = 30,
		[67] = 31,
		[69] = 32,
		[71] = 33,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[113] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A1702C0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[37] = 24,
		[39] = 25,
		[41] = 26,
		[43] = 27,
		[45] = 28,
		[47] = 29,
		[49] = 30,
		[51] = 31,
		[53] = 32,
		[55] = 33,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[114] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285C0B0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[37] = 26,
		[38] = 6,
		[39] = 27,
		[40] = 7,
		[41] = 28,
		[42] = 8,
		[43] = 29,
		[44] = 9,
		[45] = 30,
		[46] = 10,
		[47] = 31,
		[48] = 11,
		[49] = 32,
		[50] = 12,
		[51] = 33,
		[52] = 13,
		[53] = 34,
		[54] = 14,
		[55] = 35,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[115] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A17C000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[89] = 24,
		[91] = 25,
		[93] = 26,
		[95] = 27,
		[97] = 28,
		[99] = 29,
		[101] = 30,
		[103] = 31,
		[105] = 32,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[116] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285F000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[89] = 26,
		[91] = 27,
		[93] = 28,
		[95] = 29,
		[97] = 30,
		[99] = 31,
		[101] = 32,
		[103] = 33,
		[105] = 34,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[117] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A17C000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[107] = 24,
		[108] = 14,
		[109] = 25,
		[110] = 15,
		[111] = 26,
		[112] = 16,
		[113] = 27,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[118] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285F000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[107] = 26,
		[108] = 16,
		[109] = 27,
		[110] = 17,
		[111] = 28,
		[112] = 18,
		[113] = 29,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[119] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A170BC0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[57] = 24,
		[58] = 6,
		[59] = 25,
		[60] = 7,
		[61] = 26,
		[62] = 8,
		[63] = 27,
		[64] = 9,
		[65] = 28,
		[66] = 10,
		[67] = 29,
		[68] = 11,
		[69] = 30,
		[70] = 12,
		[71] = 31,
		[89] = 32,
		[91] = 33,
		[93] = 34,
		[95] = 35,
		[97] = 36,
		[99] = 37,
		[101] = 38,
		[103] = 39,
		[105] = 40,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[120] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285C2F0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[57] = 26,
		[59] = 27,
		[61] = 28,
		[63] = 29,
		[65] = 30,
		[67] = 31,
		[69] = 32,
		[71] = 33,
		[89] = 34,
		[91] = 35,
		[93] = 36,
		[95] = 37,
		[97] = 38,
		[99] = 39,
		[101] = 40,
		[103] = 41,
		[105] = 42,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[121] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A1702F0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[37] = 24,
		[39] = 25,
		[41] = 26,
		[43] = 27,
		[45] = 28,
		[47] = 29,
		[49] = 30,
		[51] = 31,
		[53] = 32,
		[55] = 33,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[89] = 34,
		[91] = 35,
		[93] = 36,
		[95] = 37,
		[97] = 38,
		[99] = 39,
		[101] = 40,
		[103] = 41,
		[105] = 42,
		[106] = 13,
		[108] = 14,
		[110] = 15,
		[112] = 16,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[122] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_TCP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285C0BC000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[37] = 26,
		[38] = 6,
		[39] = 27,
		[40] = 7,
		[41] = 28,
		[42] = 8,
		[43] = 29,
		[44] = 9,
		[45] = 30,
		[46] = 10,
		[47] = 31,
		[48] = 11,
		[49] = 32,
		[50] = 12,
		[51] = 33,
		[52] = 13,
		[53] = 34,
		[54] = 14,
		[55] = 35,
		[89] = 36,
		[91] = 37,
		[93] = 38,
		[95] = 39,
		[97] = 40,
		[99] = 41,
		[101] = 42,
		[103] = 43,
		[105] = 44,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[123] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A170BC0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[56] = 5,
		[57] = 24,
		[58] = 6,
		[59] = 25,
		[60] = 7,
		[61] = 26,
		[62] = 8,
		[63] = 27,
		[64] = 9,
		[65] = 28,
		[66] = 10,
		[67] = 29,
		[68] = 11,
		[69] = 30,
		[70] = 12,
		[71] = 31,
		[106] = 13,
		[107] = 32,
		[108] = 14,
		[109] = 33,
		[110] = 15,
		[111] = 34,
		[112] = 16,
		[113] = 35,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[124] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV6 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285C2F0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[57] = 26,
		[59] = 27,
		[61] = 28,
		[63] = 29,
		[65] = 30,
		[67] = 31,
		[69] = 32,
		[71] = 33,
		[106] = 15,
		[107] = 34,
		[108] = 16,
		[109] = 35,
		[110] = 17,
		[111] = 36,
		[112] = 18,
		[113] = 37,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[125] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV6 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00A1702F0000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 21,
		[12] = 3,
		[13] = 22,
		[14] = 4,
		[15] = 23,
		[37] = 24,
		[39] = 25,
		[41] = 26,
		[43] = 27,
		[45] = 28,
		[47] = 29,
		[49] = 30,
		[51] = 31,
		[53] = 32,
		[55] = 33,
		[56] = 5,
		[58] = 6,
		[60] = 7,
		[62] = 8,
		[64] = 9,
		[66] = 10,
		[68] = 11,
		[70] = 12,
		[106] = 13,
		[107] = 34,
		[108] = 14,
		[109] = 35,
		[110] = 15,
		[111] = 36,
		[112] = 16,
		[113] = 37,
		[126] = 17,
		[127] = 18,
		[128] = 19,
		[129] = 20,
		},
	},
	[126] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_VXLAN |
		BNXT_ULP_HDR_BIT_I_ETH |
		BNXT_ULP_HDR_BIT_I_IPV4 |
		BNXT_ULP_HDR_BIT_I_UDP |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB00285C0BC000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 1,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[11] = 23,
		[12] = 3,
		[13] = 24,
		[14] = 4,
		[15] = 25,
		[36] = 5,
		[37] = 26,
		[38] = 6,
		[39] = 27,
		[40] = 7,
		[41] = 28,
		[42] = 8,
		[43] = 29,
		[44] = 9,
		[45] = 30,
		[46] = 10,
		[47] = 31,
		[48] = 11,
		[49] = 32,
		[50] = 12,
		[51] = 33,
		[52] = 13,
		[53] = 34,
		[54] = 14,
		[55] = 35,
		[106] = 15,
		[107] = 36,
		[108] = 16,
		[109] = 37,
		[110] = 17,
		[111] = 38,
		[112] = 18,
		[113] = 39,
		[126] = 19,
		[127] = 20,
		[128] = 21,
		[129] = 22,
		},
	},
	[127] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_O_ETH |
		BNXT_ULP_HDR_BIT_O_IPV4 |
		BNXT_ULP_HDR_BIT_O_UDP |
		BNXT_ULP_HDR_BIT_T_GENEVE |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0xB002800000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 2,
	.field_list = {
		[1] = 1,
		[10] = 2,
		[12] = 3,
		[14] = 4,
		[36] = 5,
		[38] = 6,
		[40] = 7,
		[42] = 8,
		[44] = 9,
		[46] = 10,
		[48] = 11,
		[50] = 12,
		[52] = 13,
		[54] = 14,
		[106] = 15,
		[108] = 16,
		[110] = 17,
		[112] = 18,
		},
	},
	[128] = {
	.app_id = 0,
	.hdr_bitmap = { .bits =
		BNXT_ULP_HDR_BIT_SVIF |
		BNXT_ULP_FLOW_DIR_BITMASK_EGR },
	.field_man_bitmap = 0x0,
	.field_opt_bitmap = 0x8000000000000000,
	.field_exclude_bitmap = 0x0,
	.class_tid = 2,
	.flow_pattern_id = 3,
	.field_list = {
		[1] = 1,
		},
	}
};

